@@ -27,7 +27,6 @@
 #include <string.h>
 
 #include <glib.h>
-#include <glib/gthread.h>
 #include <gtk/gtk.h>
 
 #include "gs-theme-engine.h"
@@ -67,7 +66,7 @@ struct GSTESlideshowPrivate
 
         GSList         *filename_list;
         char           *images_location;
-        gboolean	sort_images;
+        gboolean        sort_images;
         int             window_width;
         int             window_height;
         PangoColor     *background_color;
@@ -171,7 +170,6 @@ start_fade (GSTESlideshow *show,
         if (gdk_pixbuf_get_has_alpha (pixbuf) && show->priv->background_color) {
                 GdkPixbuf *colored;
                 guint32    color;
-                GdkPixmap *pixmap;
 
                 color = (show->priv->background_color->red << 16)
                         + (show->priv->background_color->green / 256 << 8)
@@ -183,13 +181,10 @@ start_fade (GSTESlideshow *show,
                                                              256,
                                                              color,
                                                              color);
-                pixmap = gdk_pixmap_new (NULL, ph, pw,  gdk_visual_get_system ()->depth);
-
-                gdk_draw_pixbuf (pixmap, NULL, colored, 0, 0, 0, 0, -1, -1, GDK_RGB_DITHER_MAX, 0, 0);
-                gdk_pixbuf_get_from_drawable (pixbuf, pixmap, NULL, 0, 0, 0, 0, -1, -1);
-
-                g_object_unref (pixmap);
-
+                gdk_pixbuf_copy_area (colored, 0, 0,
+                                      gdk_pixbuf_get_width (colored),
+                                      gdk_pixbuf_get_height (colored),
+                                      pixbuf, 0, 0);
                 g_object_unref(colored);
         }
 
@@ -252,7 +247,7 @@ update_display (GSTESlideshow *show)
                                show->priv->background_color->green / 65535.0,
                                show->priv->background_color->blue / 65535.0, show->priv->alpha2);
                 } else {
-	                cairo_set_source_rgba (cr, 0.0, 0.0, 0.0, show->priv->alpha2);
+                        cairo_set_source_rgba (cr, 0.0, 0.0, 0.0, show->priv->alpha2);
                 }
                 cairo_fill (cr);
                 /* left (excluding what's covered by top and bottom) */
@@ -264,7 +259,7 @@ update_display (GSTESlideshow *show)
                                show->priv->background_color->green / 65535.0,
                                show->priv->background_color->blue / 65535.0, show->priv->alpha2);
                 } else {
-	                cairo_set_source_rgba (cr, 0.0, 0.0, 0.0, show->priv->alpha2);
+                        cairo_set_source_rgba (cr, 0.0, 0.0, 0.0, show->priv->alpha2);
                 }
                 cairo_fill (cr);
                 /* bottom */
@@ -275,7 +270,7 @@ update_display (GSTESlideshow *show)
                                show->priv->background_color->green / 65535.0,
                                show->priv->background_color->blue / 65535.0, show->priv->alpha2);
                 } else {
-	                cairo_set_source_rgba (cr, 0.0, 0.0, 0.0, show->priv->alpha2);
+                        cairo_set_source_rgba (cr, 0.0, 0.0, 0.0, show->priv->alpha2);
                 }
                 cairo_fill (cr);
                 /* right (excluding what's covered by top and bottom) */
@@ -288,7 +283,7 @@ update_display (GSTESlideshow *show)
                                show->priv->background_color->green / 65535.0,
                                show->priv->background_color->blue / 65535.0, show->priv->alpha2);
                 } else {
-	                cairo_set_source_rgba (cr, 0.0, 0.0, 0.0, show->priv->alpha2);
+                        cairo_set_source_rgba (cr, 0.0, 0.0, 0.0, show->priv->alpha2);
                 }
                 cairo_fill (cr);
 
@@ -306,18 +301,7 @@ update_display (GSTESlideshow *show)
 
         cairo_destroy (cr);
 
-        /* paint the image buffer into the window */
-        cr = gdk_cairo_create (GTK_WIDGET (show)->window);
-
-        cairo_set_source_surface (cr, show->priv->surf, 0, 0);
-
-        gs_theme_engine_profile_start ("paint surface to window");
-        cairo_paint (cr);
-        gs_theme_engine_profile_end ("paint surface to window");
-
-        cairo_destroy (cr);
-
-        gs_theme_engine_profile_end ("end");
+        gtk_widget_queue_draw (GTK_WIDGET (show));
 }
 
 static gboolean
@@ -468,9 +452,11 @@ scale_pixbuf (GdkPixbuf *pixbuf,
 
         /* always scale down, allow to disable scaling up */
         if (scale_factor < 1.0 || !no_stretch_hint) {
-                int scale_x = (int) (pw * scale_factor);
-                int scale_y = (int) (ph * scale_factor);
+                int scale_x;
+                int scale_y;
 
+                scale_x = (int) (pw * scale_factor);
+                scale_y = (int) (ph * scale_factor);
                 return gdk_pixbuf_scale_simple (pixbuf,
                                                 scale_x,
                                                 scale_y,
@@ -495,6 +481,12 @@ add_files_to_list (GSList    **list,
 
         while ((d_name = g_dir_read_name (d)) != NULL) {
                 char *path;
+
+                /* skip hidden files */
+                if (d_name[0] == '.') {
+                        continue;
+                }
+
                 path = g_build_filename (base, d_name, NULL);
                 if (g_file_test (path, G_FILE_TEST_IS_DIR)) {
                         add_files_to_list (list, path);
@@ -540,10 +532,11 @@ get_pixbuf_from_local_dir (GSTESlideshow *show,
         }
 
         if (show->priv->filename_list == NULL) {
+                return NULL;
+        } else {
                 if (show->priv->sort_images) {
                         show->priv->filename_list = g_slist_sort (show->priv->filename_list, gste_strcmp_compare_func);
                 }
-                return NULL;
         }
 
         /* get a random filename if needed */
@@ -636,7 +629,7 @@ op_load_image (GSTESlideshow *show,
         g_async_queue_push_unlocked (show->priv->results_q, op_result);
 
         if (show->priv->results_pull_id == 0) {
-		show->priv->results_pull_id = g_idle_add_full (G_PRIORITY_HIGH_IDLE,
+                show->priv->results_pull_id = g_idle_add_full (G_PRIORITY_HIGH_IDLE,
                                                                (GSourceFunc)results_pull_func,
                                                                show, NULL);
         }
@@ -737,9 +730,9 @@ gste_slideshow_set_property (GObject            *object,
         case PROP_SOLID_BACKGROUND:
                 gste_slideshow_set_background_color (self, g_value_get_string (value));
                 break;
-	case PROP_NO_STRETCH_HINT:
-		gste_slideshow_set_no_stretch_hint (self, g_value_get_boolean (value));
-		break;
+        case PROP_NO_STRETCH_HINT:
+                gste_slideshow_set_no_stretch_hint (self, g_value_get_boolean (value));
+                break;
         default:
                 G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
                 break;
@@ -760,9 +753,9 @@ gste_slideshow_get_property (GObject            *object,
         case PROP_IMAGES_LOCATION:
                 g_value_set_string (value, self->priv->images_location);
                 break;
-	case PROP_SORT_IMAGES:
-		g_value_set_boolean (value, self->priv->sort_images);
-		break;
+        case PROP_SORT_IMAGES:
+                g_value_set_boolean (value, self->priv->sort_images);
+                break;
         case PROP_SOLID_BACKGROUND:
                 {
                         char *color = NULL;
@@ -771,9 +764,9 @@ gste_slideshow_get_property (GObject            *object,
                         g_free (color);
                         break;
                 }
-	case PROP_NO_STRETCH_HINT:
-		g_value_set_boolean (value, self->priv->no_stretch_hint);
-		break;
+        case PROP_NO_STRETCH_HINT:
+                g_value_set_boolean (value, self->priv->no_stretch_hint);
+                break;
         default:
                 G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
                 break;
@@ -802,19 +795,22 @@ gste_slideshow_real_show (GtkWidget *widget)
 }
 
 static gboolean
-gste_slideshow_real_expose (GtkWidget      *widget,
-                            GdkEventExpose *event)
+gste_slideshow_real_draw (GtkWidget      *widget,
+                          cairo_t        *cr)
 {
         GSTESlideshow *show = GSTE_SLIDESHOW (widget);
-        gboolean       handled = FALSE;
 
-        update_display (show);
-
-        if (GTK_WIDGET_CLASS (parent_class)->expose_event) {
-                handled = GTK_WIDGET_CLASS (parent_class)->expose_event (widget, event);
+        if (GTK_WIDGET_CLASS (parent_class)->draw) {
+                GTK_WIDGET_CLASS (parent_class)->draw (widget, cr);
         }
 
-        return handled;
+        cairo_set_source_surface (cr, show->priv->surf, 0, 0);
+
+        gs_theme_engine_profile_start ("paint surface to window");
+        cairo_paint (cr);
+        gs_theme_engine_profile_end ("paint surface to window");
+
+        return TRUE;
 }
 
 static gboolean
@@ -838,7 +834,7 @@ gste_slideshow_real_configure (GtkWidget         *widget,
                 cairo_surface_destroy (show->priv->surf);
         }
 
-        cr = gdk_cairo_create (widget->window);
+        cr = gdk_cairo_create (gtk_widget_get_window (widget));
         show->priv->surf = cairo_surface_create_similar (cairo_get_target (cr),
                                                          CAIRO_CONTENT_COLOR,
                                                          show->priv->window_width,
@@ -868,7 +864,7 @@ gste_slideshow_class_init (GSTESlideshowClass *klass)
         object_class->set_property = gste_slideshow_set_property;
 
         widget_class->show = gste_slideshow_real_show;
-        widget_class->expose_event = gste_slideshow_real_expose;
+        widget_class->draw = gste_slideshow_real_draw;
         widget_class->configure_event = gste_slideshow_real_configure;
 
         g_type_class_add_private (klass, sizeof (GSTESlideshowPrivate));
@@ -903,21 +899,6 @@ gste_slideshow_class_init (GSTESlideshowClass *klass)
                                                                G_PARAM_READWRITE));
 }
 
-static void
-set_colormap (GtkWidget *widget)
-{
-        GdkScreen   *screen;
-        GdkColormap *colormap;
-
-        screen = gtk_widget_get_screen (widget);
-        colormap = gdk_screen_get_rgba_colormap (screen);
-        if (colormap == NULL) {
-                colormap = gdk_screen_get_rgb_colormap (screen);
-        }
-
-        gtk_widget_set_colormap (widget, colormap);
-}
-
 static void
 gste_slideshow_init (GSTESlideshow *show)
 {
@@ -938,8 +919,6 @@ gste_slideshow_init (GSTESlideshow *show)
                 g_error_free (error);
                 exit (-1);
         }
-
-        set_colormap (GTK_WIDGET (show));
 }
 
 static void
@@ -980,10 +959,10 @@ gste_slideshow_finalize (GObject *object)
         g_free (show->priv->images_location);
         show->priv->images_location = NULL;
 
-	if (show->priv->background_color) {
-		g_slice_free (PangoColor, show->priv->background_color);
-		show->priv->background_color = NULL;
-	}
+        if (show->priv->background_color) {
+                g_slice_free (PangoColor, show->priv->background_color);
+                show->priv->background_color = NULL;
+        }
 
         if (show->priv->timer != NULL) {
                 g_timer_destroy (show->priv->timer);
